package com.perforce.hive;

import com.perforce.hive.common.ClientFactory;
import com.perforce.hive.common.ClientPooledObject;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.p4java.client.IClient;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.NoSuchElementException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

public class TestPool {

	private static GenericObjectPool<ClientPooledObject> pool;

	static {
		GenericObjectPoolConfig config = new GenericObjectPoolConfig();
		config.setMaxTotal(3);
		pool = new GenericObjectPool<>(new ClientFactory(), config);
	}

	@BeforeClass
	public static void beforeClass() throws Exception {
		UnitServer server = new UnitServer("p4_root");
		server.clean();
		server.extract("depots.tar.gz");
		server.restore("checkpoint");
		server.upgrade();

		// load Connection configurations and set rsh port
		ConnectionConfig.get().load("test-core");
		String rsh = server.getRshPort();
		String key = "com.perforce.hive.p4port";
		ConnectionConfig.get().set(key, rsh);
	}

	@Test(expected = NoSuchElementException.class)
	public void consumeWithTimeout() throws Exception {
		int maxWait = 100;

		IClient client1 = pool.borrowObject(maxWait).getObject();
		assertNotNull(client1);
		IClient client2 = pool.borrowObject(maxWait).getObject();
		assertNotNull(client2);
		IClient client3 = pool.borrowObject(maxWait).getObject();
		assertNotNull(client3);

		IClient client4 = pool.borrowObject(maxWait).getObject();
		assertNull(client4);
	}

	@Test
	public void consumeThenFreeWithTimeout() throws Exception {
		int maxWait = 100;

		IClient client1 = pool.borrowObject(maxWait).getObject();
		IClient client2 = pool.borrowObject(maxWait).getObject();
		IClient client3 = pool.borrowObject(maxWait).getObject();

		pool.returnObject(new ClientPooledObject(client2));

		IClient client4 = pool.borrowObject(maxWait).getObject();
		assertNotNull(client4);
		assertEquals("p4am.maven.2", client4.getName());
	}
}
